/** Main file for USB storage key **/
/** Memory write and read test    **/
/** Write to blocks and verify    **/

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdint.h>

#include "io.h"
#include "spi.h"
#include "AT45DB641E.h"

// Constants

#define LED_NB_START    10
#define NB_BLOCKS_FLASH	128

#define MAX_LED		1

#define LED		0

#define PORTB_OUTPUT	0x40
#define PORTC_OUTPUT	0x00
#define PORTD_OUTPUT	0x00

unsigned char led1[]={0x40,0x00,0x00};

unsigned char *leds[]={led1};

#define MEM_RESET_DDR	DDRB
#define MEM_RESET_PORT	PORTB
#define MEM_RESET_PIN	5

#define MEM1_DDR	DDRD
#define MEM1_PORT	PORTD
#define MEM1_PIN	0

#define MEM2_DDR	DDRB
#define MEM2_PORT	PORTB
#define MEM2_PIN	4

#define BLOCK_COUNT	(1L<<15)
#define BLOCK_SIZE	256

#ifdef DEBUG
void blink_byte(uint8_t byte,int led){
int i;
for(i=0;i<8;i++){
  int nb=(byte&0x01)?2:1;
  int j;
  for(j=0;j<nb;j++){
    output_set(leds,led); _delay_ms(100);
    output_unset(leds,led); _delay_ms(100);
    }
  _delay_ms(1000);
  byte >>= 1;
  }
}
#endif

void blink_start(int led){
int i;
for(i=0;i<LED_NB_START;i++){
  output_set(leds,led); _delay_ms(50);
  output_unset(leds,led); _delay_ms(50);
  }
}

void blink_bool(uint8_t b,int led){
int i,nb;
if(b) nb=5; else nb=1;
for(i=0;i<nb;i++){
  output_set(leds,led); _delay_ms(100);
  output_unset(leds,led); _delay_ms(100);
  }
}

int main(void)
{
unsigned char omasks[]={PORTB_OUTPUT,PORTC_OUTPUT,PORTD_OUTPUT};
outputs_init(omasks);

MEM1_DDR |= (1<<MEM1_PIN);
MEM2_DDR |= (1<<MEM2_PIN);
MEM1_PORT |= (1<<MEM1_PIN);
MEM2_PORT |= (1<<MEM2_PIN);
MEM_RESET_DDR |= (1<<MEM_RESET_PIN);
MEM_RESET_PORT |= (1<<MEM_RESET_PIN);
spi_init();

blink_start(LED);
_delay_ms(1000);
AT45DB641E_page_size(&MEM1_PORT,MEM1_PIN,256);
AT45DB641E_page_size(&MEM2_PORT,MEM2_PIN,256);
long int i;
int j;
for(i=0;i<BLOCK_COUNT;i++){
  if(i%NB_BLOCKS_FLASH==0) output_toogle(leds,LED);
  AT45DB641E_write_buffer(&MEM1_PORT,MEM1_PIN,NULL,0,SEQ_START);
  for(j=0;j<BLOCK_SIZE;j++){
    unsigned char byte=i&0xff;
    AT45DB641E_write_buffer(&MEM1_PORT,MEM1_PIN,&byte,1,0);
    }
  AT45DB641E_write_buffer(&MEM1_PORT,MEM1_PIN,NULL,0,SEQ_STOP);
  AT45DB641E_write_page(&MEM1_PORT,MEM1_PIN,i);
  }
unsigned char ok=1;
for(i=0;i<BLOCK_COUNT;i++){
  if(i%NB_BLOCKS_FLASH==0) output_toogle(leds,LED);
  AT45DB641E_read_page(&MEM1_PORT,MEM1_PIN,i,NULL,0,SEQ_START);
  for(j=0;j<BLOCK_SIZE;j++){
    unsigned char byte;
    AT45DB641E_read_page(&MEM1_PORT,MEM1_PIN,0,&byte,1,0);
    if(byte!=(i&0xff)){ ok=0; break; }
    }
  if(j!=BLOCK_SIZE) break;
  AT45DB641E_read_page(&MEM1_PORT,MEM1_PIN,i,NULL,0,SEQ_STOP);
  }
_delay_ms(1000);

while(1){
  blink_bool(ok,LED);
  _delay_ms(5000);
  }

return 0;
}
